@@ -64,7 +64,8 @@ struct http_parser {
   } len;
 };
 
-static ID id_set_backtrace;
+static ID id_set_backtrace, id_is_chunked_p;
+static VALUE cHttpParser;
 
 #ifdef HAVE_RB_HASH_CLEAR /* Ruby >= 2.0 */
 #  define my_hash_clear(h) (void)rb_hash_clear(h)
@@ -222,6 +223,19 @@ static void write_cont_value(struct http_parser *hp,
   rb_str_buf_cat(hp->cont, vptr, end + 1);
 }
 
+static int is_chunked(VALUE v)
+{
+  /* common case first */
+  if (STR_CSTR_CASE_EQ(v, "chunked"))
+    return 1;
+
+  /*
+   * call Ruby function in unicorn/http_request.rb to deal with unlikely
+   * comma-delimited case
+   */
+  return rb_funcall(cHttpParser, id_is_chunked_p, 1, v) != Qfalse;
+}
+
 static void write_value(struct http_parser *hp,
                         const char *buffer, const char *p)
 {
@@ -248,7 +262,9 @@ static void write_value(struct http_parser *hp,
     f = uncommon_field(field, flen);
   } else if (f == g_http_connection) {
     hp_keepalive_connection(hp, v);
-  } else if (f == g_content_length) {
+  } else if (f == g_content_length && !HP_FL_TEST(hp, CHUNKED)) {
+    if (hp->len.content)
+      parser_raise(eHttpParserError, "Content-Length already set");
     hp->len.content = parse_length(RSTRING_PTR(v), RSTRING_LEN(v));
     if (hp->len.content < 0)
       parser_raise(eHttpParserError, "invalid Content-Length");
@@ -256,9 +272,30 @@ static void write_value(struct http_parser *hp,
       HP_FL_SET(hp, HASBODY);
     hp_invalid_if_trailer(hp);
   } else if (f == g_http_transfer_encoding) {
-    if (STR_CSTR_CASE_EQ(v, "chunked")) {
+    if (is_chunked(v)) {
+      if (HP_FL_TEST(hp, CHUNKED))
+        /*
+         * RFC 7230 3.3.1:
+         * A sender MUST NOT apply chunked more than once to a message body
+         * (i.e., chunking an already chunked message is not allowed).
+         */
+        parser_raise(eHttpParserError, "Transfer-Encoding double chunked");
+
       HP_FL_SET(hp, CHUNKED);
       HP_FL_SET(hp, HASBODY);
+
+      /* RFC 7230 3.3.3, 3: favor chunked if Content-Length exists */
+      hp->len.content = 0;
+    } else if (HP_FL_TEST(hp, CHUNKED)) {
+      /*
+       * RFC 7230 3.3.3, point 3 states:
+       * If a Transfer-Encoding header field is present in a request and
+       * the chunked transfer coding is not the final encoding, the
+       * message body length cannot be determined reliably; the server
+       * MUST respond with the 400 (Bad Request) status code and then
+       * close the connection.
+       */
+      parser_raise(eHttpParserError, "invalid Transfer-Encoding");
     }
     hp_invalid_if_trailer(hp);
   } else if (f == g_http_trailer) {
@@ -287,12 +324,12 @@ static void write_value(struct http_parser *hp,
 /** Machine **/
 
 
-#line 394 "unicorn_http.rl"
+#line 431 "unicorn_http.rl"
 
 
 /** Data **/
 
-#line 296 "unicorn_http.c"
+#line 333 "unicorn_http.c"
 static const int http_parser_start = 1;
 static const int http_parser_first_final = 122;
 static const int http_parser_error = 0;
@@ -303,7 +340,7 @@ static const int http_parser_en_Trailers = 114;
 static const int http_parser_en_main = 1;
 
 
-#line 398 "unicorn_http.rl"
+#line 435 "unicorn_http.rl"
 
 static void http_parser_init(struct http_parser *hp)
 {
@@ -316,12 +353,12 @@ static void http_parser_init(struct http_parser *hp)
   hp->len.content = 0;
   hp->cont = Qfalse; /* zero on MRI, should be optimized away by above */
   
-#line 320 "unicorn_http.c"
+#line 357 "unicorn_http.c"
 	{
 	cs = http_parser_start;
 	}
 
-#line 410 "unicorn_http.rl"
+#line 447 "unicorn_http.rl"
   hp->cs = cs;
 }
 
@@ -349,7 +386,7 @@ http_parser_execute(struct http_parser *hp, char *buffer, size_t len)
     goto skip_chunk_data_hack;
   }
   
-#line 353 "unicorn_http.c"
+#line 390 "unicorn_http.c"
 	{
 	if ( p == pe )
 		goto _test_eof;
@@ -384,14 +421,14 @@ st0:
 cs = 0;
 	goto _out;
 tr0:
-#line 290 "unicorn_http.rl"
+#line 327 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st2;
 st2:
 	if ( ++p == pe )
 		goto _test_eof2;
 case 2:
-#line 395 "unicorn_http.c"
+#line 432 "unicorn_http.c"
 	switch( (*p) ) {
 		case 32: goto tr3;
 		case 33: goto st49;
@@ -417,14 +454,14 @@ case 2:
 		goto st49;
 	goto st0;
 tr3:
-#line 299 "unicorn_http.rl"
+#line 336 "unicorn_http.rl"
 	{ request_method(hp, PTR_TO(mark), LEN(mark, p)); }
 	goto st3;
 st3:
 	if ( ++p == pe )
 		goto _test_eof3;
 case 3:
-#line 428 "unicorn_http.c"
+#line 465 "unicorn_http.c"
 	switch( (*p) ) {
 		case 42: goto tr5;
 		case 47: goto tr6;
@@ -433,21 +470,21 @@ case 3:
 	}
 	goto st0;
 tr5:
-#line 290 "unicorn_http.rl"
+#line 327 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st4;
 st4:
 	if ( ++p == pe )
 		goto _test_eof4;
 case 4:
-#line 444 "unicorn_http.c"
+#line 481 "unicorn_http.c"
 	switch( (*p) ) {
 		case 32: goto tr8;
 		case 35: goto tr9;
 	}
 	goto st0;
 tr8:
-#line 304 "unicorn_http.rl"
+#line 341 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -465,23 +502,23 @@ tr8:
   }
 	goto st5;
 tr42:
-#line 290 "unicorn_http.rl"
+#line 327 "unicorn_http.rl"
 	{MARK(mark, p); }
-#line 319 "unicorn_http.rl"
+#line 356 "unicorn_http.rl"
 	{
     VALIDATE_MAX_URI_LENGTH(LEN(mark, p), FRAGMENT);
     rb_hash_aset(hp->env, g_fragment, STR_NEW(mark, p));
   }
 	goto st5;
 tr45:
-#line 319 "unicorn_http.rl"
+#line 356 "unicorn_http.rl"
 	{
     VALIDATE_MAX_URI_LENGTH(LEN(mark, p), FRAGMENT);
     rb_hash_aset(hp->env, g_fragment, STR_NEW(mark, p));
   }
 	goto st5;
 tr49:
-#line 329 "unicorn_http.rl"
+#line 366 "unicorn_http.rl"
 	{
     VALUE val;
 
@@ -492,7 +529,7 @@ tr49:
     if (!STR_CSTR_EQ(val, "*"))
       rb_hash_aset(hp->env, g_path_info, val);
   }
-#line 304 "unicorn_http.rl"
+#line 341 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -510,14 +547,14 @@ tr49:
   }
 	goto st5;
 tr55:
-#line 323 "unicorn_http.rl"
+#line 360 "unicorn_http.rl"
 	{MARK(start.query, p); }
-#line 324 "unicorn_http.rl"
+#line 361 "unicorn_http.rl"
 	{
     VALIDATE_MAX_URI_LENGTH(LEN(start.query, p), QUERY_STRING);
     rb_hash_aset(hp->env, g_query_string, STR_NEW(start.query, p));
   }
-#line 304 "unicorn_http.rl"
+#line 341 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -535,12 +572,12 @@ tr55:
   }
 	goto st5;
 tr59:
-#line 324 "unicorn_http.rl"
+#line 361 "unicorn_http.rl"
 	{
     VALIDATE_MAX_URI_LENGTH(LEN(start.query, p), QUERY_STRING);
     rb_hash_aset(hp->env, g_query_string, STR_NEW(start.query, p));
   }
-#line 304 "unicorn_http.rl"
+#line 341 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -561,19 +598,19 @@ st5:
 	if ( ++p == pe )
 		goto _test_eof5;
 case 5:
-#line 565 "unicorn_http.c"
+#line 602 "unicorn_http.c"
 	if ( (*p) == 72 )
 		goto tr10;
 	goto st0;
 tr10:
-#line 290 "unicorn_http.rl"
+#line 327 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st6;
 st6:
 	if ( ++p == pe )
 		goto _test_eof6;
 case 6:
-#line 577 "unicorn_http.c"
+#line 614 "unicorn_http.c"
 	if ( (*p) == 84 )
 		goto st7;
 	goto st0;
@@ -633,34 +670,34 @@ case 13:
 		goto st13;
 	goto st0;
 tr18:
-#line 328 "unicorn_http.rl"
+#line 365 "unicorn_http.rl"
 	{ http_version(hp, PTR_TO(mark), LEN(mark, p)); }
 	goto st14;
 tr26:
-#line 296 "unicorn_http.rl"
+#line 333 "unicorn_http.rl"
 	{ MARK(mark, p); }
-#line 298 "unicorn_http.rl"
+#line 335 "unicorn_http.rl"
 	{ write_cont_value(hp, buffer, p); }
 	goto st14;
 tr29:
-#line 298 "unicorn_http.rl"
+#line 335 "unicorn_http.rl"
 	{ write_cont_value(hp, buffer, p); }
 	goto st14;
 tr36:
-#line 296 "unicorn_http.rl"
+#line 333 "unicorn_http.rl"
 	{ MARK(mark, p); }
-#line 297 "unicorn_http.rl"
+#line 334 "unicorn_http.rl"
 	{ write_value(hp, buffer, p); }
 	goto st14;
 tr39:
-#line 297 "unicorn_http.rl"
+#line 334 "unicorn_http.rl"
 	{ write_value(hp, buffer, p); }
 	goto st14;
 st14:
 	if ( ++p == pe )
 		goto _test_eof14;
 case 14:
-#line 664 "unicorn_http.c"
+#line 701 "unicorn_http.c"
 	switch( (*p) ) {
 		case 9: goto st15;
 		case 10: goto tr21;
@@ -689,14 +726,14 @@ case 14:
 		goto tr23;
 	goto st0;
 tr25:
-#line 296 "unicorn_http.rl"
+#line 333 "unicorn_http.rl"
 	{ MARK(mark, p); }
 	goto st15;
 st15:
 	if ( ++p == pe )
 		goto _test_eof15;
 case 15:
-#line 700 "unicorn_http.c"
+#line 737 "unicorn_http.c"
 	switch( (*p) ) {
 		case 9: goto tr25;
 		case 10: goto tr26;
@@ -708,14 +745,14 @@ case 15:
 		goto st0;
 	goto tr24;
 tr24:
-#line 296 "unicorn_http.rl"
+#line 333 "unicorn_http.rl"
 	{ MARK(mark, p); }
 	goto st16;
 st16:
 	if ( ++p == pe )
 		goto _test_eof16;
 case 16:
-#line 719 "unicorn_http.c"
+#line 756 "unicorn_http.c"
 	switch( (*p) ) {
 		case 10: goto tr29;
 		case 13: goto tr30;
@@ -728,39 +765,39 @@ case 16:
 		goto st0;
 	goto st16;
 tr19:
-#line 328 "unicorn_http.rl"
+#line 365 "unicorn_http.rl"
 	{ http_version(hp, PTR_TO(mark), LEN(mark, p)); }
 	goto st17;
 tr27:
-#line 296 "unicorn_http.rl"
+#line 333 "unicorn_http.rl"
 	{ MARK(mark, p); }
-#line 298 "unicorn_http.rl"
+#line 335 "unicorn_http.rl"
 	{ write_cont_value(hp, buffer, p); }
 	goto st17;
 tr30:
-#line 298 "unicorn_http.rl"
+#line 335 "unicorn_http.rl"
 	{ write_cont_value(hp, buffer, p); }
 	goto st17;
 tr37:
-#line 296 "unicorn_http.rl"
+#line 333 "unicorn_http.rl"
 	{ MARK(mark, p); }
-#line 297 "unicorn_http.rl"
+#line 334 "unicorn_http.rl"
 	{ write_value(hp, buffer, p); }
 	goto st17;
 tr40:
-#line 297 "unicorn_http.rl"
+#line 334 "unicorn_http.rl"
 	{ write_value(hp, buffer, p); }
 	goto st17;
 st17:
 	if ( ++p == pe )
 		goto _test_eof17;
 case 17:
-#line 759 "unicorn_http.c"
+#line 796 "unicorn_http.c"
 	if ( (*p) == 10 )
 		goto st14;
 	goto st0;
 tr21:
-#line 344 "unicorn_http.rl"
+#line 381 "unicorn_http.rl"
 	{
     finalize_header(hp);
 
@@ -781,7 +818,7 @@ tr21:
   }
 	goto st122;
 tr104:
-#line 304 "unicorn_http.rl"
+#line 341 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -797,7 +834,7 @@ tr104:
       rb_hash_aset(hp->env, g_request_path, str);
     }
   }
-#line 344 "unicorn_http.rl"
+#line 381 "unicorn_http.rl"
 	{
     finalize_header(hp);
 
@@ -818,14 +855,14 @@ tr104:
   }
 	goto st122;
 tr108:
-#line 290 "unicorn_http.rl"
+#line 327 "unicorn_http.rl"
 	{MARK(mark, p); }
-#line 319 "unicorn_http.rl"
+#line 356 "unicorn_http.rl"
 	{
     VALIDATE_MAX_URI_LENGTH(LEN(mark, p), FRAGMENT);
     rb_hash_aset(hp->env, g_fragment, STR_NEW(mark, p));
   }
-#line 344 "unicorn_http.rl"
+#line 381 "unicorn_http.rl"
 	{
     finalize_header(hp);
 
@@ -846,12 +883,12 @@ tr108:
   }
 	goto st122;
 tr112:
-#line 319 "unicorn_http.rl"
+#line 356 "unicorn_http.rl"
 	{
     VALIDATE_MAX_URI_LENGTH(LEN(mark, p), FRAGMENT);
     rb_hash_aset(hp->env, g_fragment, STR_NEW(mark, p));
   }
-#line 344 "unicorn_http.rl"
+#line 381 "unicorn_http.rl"
 	{
     finalize_header(hp);
 
@@ -872,7 +909,7 @@ tr112:
   }
 	goto st122;
 tr117:
-#line 329 "unicorn_http.rl"
+#line 366 "unicorn_http.rl"
 	{
     VALUE val;
 
@@ -883,7 +920,7 @@ tr117:
     if (!STR_CSTR_EQ(val, "*"))
       rb_hash_aset(hp->env, g_path_info, val);
   }
-#line 304 "unicorn_http.rl"
+#line 341 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -899,7 +936,7 @@ tr117:
       rb_hash_aset(hp->env, g_request_path, str);
     }
   }
-#line 344 "unicorn_http.rl"
+#line 381 "unicorn_http.rl"
 	{
     finalize_header(hp);
 
@@ -920,14 +957,14 @@ tr117:
   }
 	goto st122;
 tr124:
-#line 323 "unicorn_http.rl"
+#line 360 "unicorn_http.rl"
 	{MARK(start.query, p); }
-#line 324 "unicorn_http.rl"
+#line 361 "unicorn_http.rl"
 	{
     VALIDATE_MAX_URI_LENGTH(LEN(start.query, p), QUERY_STRING);
     rb_hash_aset(hp->env, g_query_string, STR_NEW(start.query, p));
   }
-#line 304 "unicorn_http.rl"
+#line 341 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -943,7 +980,7 @@ tr124:
       rb_hash_aset(hp->env, g_request_path, str);
     }
   }
-#line 344 "unicorn_http.rl"
+#line 381 "unicorn_http.rl"
 	{
     finalize_header(hp);
 
@@ -964,12 +1001,12 @@ tr124:
   }
 	goto st122;
 tr129:
-#line 324 "unicorn_http.rl"
+#line 361 "unicorn_http.rl"
 	{
     VALIDATE_MAX_URI_LENGTH(LEN(start.query, p), QUERY_STRING);
     rb_hash_aset(hp->env, g_query_string, STR_NEW(start.query, p));
   }
-#line 304 "unicorn_http.rl"
+#line 341 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -985,7 +1022,7 @@ tr129:
       rb_hash_aset(hp->env, g_request_path, str);
     }
   }
-#line 344 "unicorn_http.rl"
+#line 381 "unicorn_http.rl"
 	{
     finalize_header(hp);
 
@@ -1009,10 +1046,10 @@ st122:
 	if ( ++p == pe )
 		goto _test_eof122;
 case 122:
-#line 1013 "unicorn_http.c"
+#line 1050 "unicorn_http.c"
 	goto st0;
 tr105:
-#line 304 "unicorn_http.rl"
+#line 341 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -1030,23 +1067,23 @@ tr105:
   }
 	goto st18;
 tr109:
-#line 290 "unicorn_http.rl"
+#line 327 "unicorn_http.rl"
 	{MARK(mark, p); }
-#line 319 "unicorn_http.rl"
+#line 356 "unicorn_http.rl"
 	{
     VALIDATE_MAX_URI_LENGTH(LEN(mark, p), FRAGMENT);
     rb_hash_aset(hp->env, g_fragment, STR_NEW(mark, p));
   }
 	goto st18;
 tr113:
-#line 319 "unicorn_http.rl"
+#line 356 "unicorn_http.rl"
 	{
     VALIDATE_MAX_URI_LENGTH(LEN(mark, p), FRAGMENT);
     rb_hash_aset(hp->env, g_fragment, STR_NEW(mark, p));
   }
 	goto st18;
 tr118:
-#line 329 "unicorn_http.rl"
+#line 366 "unicorn_http.rl"
 	{
     VALUE val;
 
@@ -1057,7 +1094,7 @@ tr118:
     if (!STR_CSTR_EQ(val, "*"))
       rb_hash_aset(hp->env, g_path_info, val);
   }
-#line 304 "unicorn_http.rl"
+#line 341 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -1075,14 +1112,14 @@ tr118:
   }
 	goto st18;
 tr125:
-#line 323 "unicorn_http.rl"
+#line 360 "unicorn_http.rl"
 	{MARK(start.query, p); }
-#line 324 "unicorn_http.rl"
+#line 361 "unicorn_http.rl"
 	{
     VALIDATE_MAX_URI_LENGTH(LEN(start.query, p), QUERY_STRING);
     rb_hash_aset(hp->env, g_query_string, STR_NEW(start.query, p));
   }
-#line 304 "unicorn_http.rl"
+#line 341 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -1100,12 +1137,12 @@ tr125:
   }
 	goto st18;
 tr130:
-#line 324 "unicorn_http.rl"
+#line 361 "unicorn_http.rl"
 	{
     VALIDATE_MAX_URI_LENGTH(LEN(start.query, p), QUERY_STRING);
     rb_hash_aset(hp->env, g_query_string, STR_NEW(start.query, p));
   }
-#line 304 "unicorn_http.rl"
+#line 341 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -1126,25 +1163,25 @@ st18:
 	if ( ++p == pe )
 		goto _test_eof18;
 case 18:
-#line 1130 "unicorn_http.c"
+#line 1167 "unicorn_http.c"
 	if ( (*p) == 10 )
 		goto tr21;
 	goto st0;
 tr23:
-#line 292 "unicorn_http.rl"
+#line 329 "unicorn_http.rl"
 	{ MARK(start.field, p); }
-#line 293 "unicorn_http.rl"
+#line 330 "unicorn_http.rl"
 	{ snake_upcase_char(deconst(p)); }
 	goto st19;
 tr32:
-#line 293 "unicorn_http.rl"
+#line 330 "unicorn_http.rl"
 	{ snake_upcase_char(deconst(p)); }
 	goto st19;
 st19:
 	if ( ++p == pe )
 		goto _test_eof19;
 case 19:
-#line 1148 "unicorn_http.c"
+#line 1185 "unicorn_http.c"
 	switch( (*p) ) {
 		case 33: goto tr32;
 		case 58: goto tr33;
@@ -1170,18 +1207,18 @@ case 19:
 		goto tr32;
 	goto st0;
 tr35:
-#line 296 "unicorn_http.rl"
+#line 333 "unicorn_http.rl"
 	{ MARK(mark, p); }
 	goto st20;
 tr33:
-#line 295 "unicorn_http.rl"
+#line 332 "unicorn_http.rl"
 	{ hp->s.field_len = LEN(start.field, p); }
 	goto st20;
 st20:
 	if ( ++p == pe )
 		goto _test_eof20;
 case 20:
-#line 1185 "unicorn_http.c"
+#line 1222 "unicorn_http.c"
 	switch( (*p) ) {
 		case 9: goto tr35;
 		case 10: goto tr36;
@@ -1193,14 +1230,14 @@ case 20:
 		goto st0;
 	goto tr34;
 tr34:
-#line 296 "unicorn_http.rl"
+#line 333 "unicorn_http.rl"
 	{ MARK(mark, p); }
 	goto st21;
 st21:
 	if ( ++p == pe )
 		goto _test_eof21;
 case 21:
-#line 1204 "unicorn_http.c"
+#line 1241 "unicorn_http.c"
 	switch( (*p) ) {
 		case 10: goto tr39;
 		case 13: goto tr40;
@@ -1213,7 +1250,7 @@ case 21:
 		goto st0;
 	goto st21;
 tr9:
-#line 304 "unicorn_http.rl"
+#line 341 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -1231,7 +1268,7 @@ tr9:
   }
 	goto st22;
 tr50:
-#line 329 "unicorn_http.rl"
+#line 366 "unicorn_http.rl"
 	{
     VALUE val;
 
@@ -1242,7 +1279,7 @@ tr50:
     if (!STR_CSTR_EQ(val, "*"))
       rb_hash_aset(hp->env, g_path_info, val);
   }
-#line 304 "unicorn_http.rl"
+#line 341 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -1260,14 +1297,14 @@ tr50:
   }
 	goto st22;
 tr56:
-#line 323 "unicorn_http.rl"
+#line 360 "unicorn_http.rl"
 	{MARK(start.query, p); }
-#line 324 "unicorn_http.rl"
+#line 361 "unicorn_http.rl"
 	{
     VALIDATE_MAX_URI_LENGTH(LEN(start.query, p), QUERY_STRING);
     rb_hash_aset(hp->env, g_query_string, STR_NEW(start.query, p));
   }
-#line 304 "unicorn_http.rl"
+#line 341 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -1285,12 +1322,12 @@ tr56:
   }
 	goto st22;
 tr60:
-#line 324 "unicorn_http.rl"
+#line 361 "unicorn_http.rl"
 	{
     VALIDATE_MAX_URI_LENGTH(LEN(start.query, p), QUERY_STRING);
     rb_hash_aset(hp->env, g_query_string, STR_NEW(start.query, p));
   }
-#line 304 "unicorn_http.rl"
+#line 341 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -1311,7 +1348,7 @@ st22:
 	if ( ++p == pe )
 		goto _test_eof22;
 case 22:
-#line 1315 "unicorn_http.c"
+#line 1352 "unicorn_http.c"
 	switch( (*p) ) {
 		case 32: goto tr42;
 		case 35: goto st0;
@@ -1322,14 +1359,14 @@ case 22:
 		goto st0;
 	goto tr41;
 tr41:
-#line 290 "unicorn_http.rl"
+#line 327 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st23;
 st23:
 	if ( ++p == pe )
 		goto _test_eof23;
 case 23:
-#line 1333 "unicorn_http.c"
+#line 1370 "unicorn_http.c"
 	switch( (*p) ) {
 		case 32: goto tr45;
 		case 35: goto st0;
@@ -1340,14 +1377,14 @@ case 23:
 		goto st0;
 	goto st23;
 tr43:
-#line 290 "unicorn_http.rl"
+#line 327 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st24;
 st24:
 	if ( ++p == pe )
 		goto _test_eof24;
 case 24:
-#line 1351 "unicorn_http.c"
+#line 1388 "unicorn_http.c"
 	if ( (*p) < 65 ) {
 		if ( 48 <= (*p) && (*p) <= 57 )
 			goto st25;
@@ -1371,20 +1408,20 @@ case 25:
 		goto st23;
 	goto st0;
 tr6:
-#line 290 "unicorn_http.rl"
+#line 327 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st26;
 tr76:
-#line 303 "unicorn_http.rl"
+#line 340 "unicorn_http.rl"
 	{ rb_hash_aset(hp->env, g_http_host, STR_NEW(mark, p)); }
-#line 290 "unicorn_http.rl"
+#line 327 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st26;
 st26:
 	if ( ++p == pe )
 		goto _test_eof26;
 case 26:
-#line 1388 "unicorn_http.c"
+#line 1425 "unicorn_http.c"
 	switch( (*p) ) {
 		case 32: goto tr49;
 		case 35: goto tr50;
@@ -1422,7 +1459,7 @@ case 28:
 		goto st26;
 	goto st0;
 tr52:
-#line 329 "unicorn_http.rl"
+#line 366 "unicorn_http.rl"
 	{
     VALUE val;
 
@@ -1438,7 +1475,7 @@ st29:
 	if ( ++p == pe )
 		goto _test_eof29;
 case 29:
-#line 1442 "unicorn_http.c"
+#line 1479 "unicorn_http.c"
 	switch( (*p) ) {
 		case 32: goto tr55;
 		case 35: goto tr56;
@@ -1449,14 +1486,14 @@ case 29:
 		goto st0;
 	goto tr54;
 tr54:
-#line 323 "unicorn_http.rl"
+#line 360 "unicorn_http.rl"
 	{MARK(start.query, p); }
 	goto st30;
 st30:
 	if ( ++p == pe )
 		goto _test_eof30;
 case 30:
-#line 1460 "unicorn_http.c"
+#line 1497 "unicorn_http.c"
 	switch( (*p) ) {
 		case 32: goto tr59;
 		case 35: goto tr60;
@@ -1467,14 +1504,14 @@ case 30:
 		goto st0;
 	goto st30;
 tr57:
-#line 323 "unicorn_http.rl"
+#line 360 "unicorn_http.rl"
 	{MARK(start.query, p); }
 	goto st31;
 st31:
 	if ( ++p == pe )
 		goto _test_eof31;
 case 31:
-#line 1478 "unicorn_http.c"
+#line 1515 "unicorn_http.c"
 	if ( (*p) < 65 ) {
 		if ( 48 <= (*p) && (*p) <= 57 )
 			goto st32;
@@ -1498,58 +1535,58 @@ case 32:
 		goto st30;
 	goto st0;
 tr7:
-#line 290 "unicorn_http.rl"
+#line 327 "unicorn_http.rl"
 	{MARK(mark, p); }
-#line 294 "unicorn_http.rl"
+#line 331 "unicorn_http.rl"
 	{ downcase_char(deconst(p)); }
 	goto st33;
 st33:
 	if ( ++p == pe )
 		goto _test_eof33;
 case 33:
-#line 1511 "unicorn_http.c"
+#line 1548 "unicorn_http.c"
 	switch( (*p) ) {
 		case 84: goto tr63;
 		case 116: goto tr63;
 	}
 	goto st0;
 tr63:
-#line 294 "unicorn_http.rl"
+#line 331 "unicorn_http.rl"
 	{ downcase_char(deconst(p)); }
 	goto st34;
 st34:
 	if ( ++p == pe )
 		goto _test_eof34;
 case 34:
-#line 1525 "unicorn_http.c"
+#line 1562 "unicorn_http.c"
 	switch( (*p) ) {
 		case 84: goto tr64;
 		case 116: goto tr64;
 	}
 	goto st0;
 tr64:
-#line 294 "unicorn_http.rl"
+#line 331 "unicorn_http.rl"
 	{ downcase_char(deconst(p)); }
 	goto st35;
 st35:
 	if ( ++p == pe )
 		goto _test_eof35;
 case 35:
-#line 1539 "unicorn_http.c"
+#line 1576 "unicorn_http.c"
 	switch( (*p) ) {
 		case 80: goto tr65;
 		case 112: goto tr65;
 	}
 	goto st0;
 tr65:
-#line 294 "unicorn_http.rl"
+#line 331 "unicorn_http.rl"
 	{ downcase_char(deconst(p)); }
 	goto st36;
 st36:
 	if ( ++p == pe )
 		goto _test_eof36;
 case 36:
-#line 1553 "unicorn_http.c"
+#line 1590 "unicorn_http.c"
 	switch( (*p) ) {
 		case 58: goto tr66;
 		case 83: goto tr67;
@@ -1557,7 +1594,7 @@ case 36:
 	}
 	goto st0;
 tr66:
-#line 300 "unicorn_http.rl"
+#line 337 "unicorn_http.rl"
 	{
     rb_hash_aset(hp->env, g_rack_url_scheme, STR_NEW(mark, p));
   }
@@ -1566,7 +1603,7 @@ st37:
 	if ( ++p == pe )
 		goto _test_eof37;
 case 37:
-#line 1570 "unicorn_http.c"
+#line 1607 "unicorn_http.c"
 	if ( (*p) == 47 )
 		goto st38;
 	goto st0;
@@ -1654,14 +1691,14 @@ case 42:
 		goto st40;
 	goto st0;
 tr72:
-#line 290 "unicorn_http.rl"
+#line 327 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st43;
 st43:
 	if ( ++p == pe )
 		goto _test_eof43;
 case 43:
-#line 1665 "unicorn_http.c"
+#line 1702 "unicorn_http.c"
 	switch( (*p) ) {
 		case 37: goto st41;
 		case 47: goto tr76;
@@ -1713,14 +1750,14 @@ case 44:
 		goto st0;
 	goto st40;
 tr73:
-#line 290 "unicorn_http.rl"
+#line 327 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st45;
 st45:
 	if ( ++p == pe )
 		goto _test_eof45;
 case 45:
-#line 1724 "unicorn_http.c"
+#line 1761 "unicorn_http.c"
 	switch( (*p) ) {
 		case 37: goto st41;
 		case 47: goto st0;
@@ -1798,14 +1835,14 @@ case 47:
 		goto st0;
 	goto st40;
 tr67:
-#line 294 "unicorn_http.rl"
+#line 331 "unicorn_http.rl"
 	{ downcase_char(deconst(p)); }
 	goto st48;
 st48:
 	if ( ++p == pe )
 		goto _test_eof48;
 case 48:
-#line 1809 "unicorn_http.c"
+#line 1846 "unicorn_http.c"
 	if ( (*p) == 58 )
 		goto tr66;
 	goto st0;
@@ -2321,14 +2358,14 @@ case 67:
 		goto tr3;
 	goto st0;
 tr2:
-#line 290 "unicorn_http.rl"
+#line 327 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st68;
 st68:
 	if ( ++p == pe )
 		goto _test_eof68;
 case 68:
-#line 2332 "unicorn_http.c"
+#line 2369 "unicorn_http.c"
 	switch( (*p) ) {
 		case 32: goto tr3;
 		case 33: goto st49;
@@ -2412,14 +2449,14 @@ case 70:
 		goto st51;
 	goto st0;
 tr100:
-#line 299 "unicorn_http.rl"
+#line 336 "unicorn_http.rl"
 	{ request_method(hp, PTR_TO(mark), LEN(mark, p)); }
 	goto st71;
 st71:
 	if ( ++p == pe )
 		goto _test_eof71;
 case 71:
-#line 2423 "unicorn_http.c"
+#line 2460 "unicorn_http.c"
 	switch( (*p) ) {
 		case 42: goto tr101;
 		case 47: goto tr102;
@@ -2428,14 +2465,14 @@ case 71:
 	}
 	goto st0;
 tr101:
-#line 290 "unicorn_http.rl"
+#line 327 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st72;
 st72:
 	if ( ++p == pe )
 		goto _test_eof72;
 case 72:
-#line 2439 "unicorn_http.c"
+#line 2476 "unicorn_http.c"
 	switch( (*p) ) {
 		case 10: goto tr104;
 		case 13: goto tr105;
@@ -2444,7 +2481,7 @@ case 72:
 	}
 	goto st0;
 tr106:
-#line 304 "unicorn_http.rl"
+#line 341 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -2462,7 +2499,7 @@ tr106:
   }
 	goto st73;
 tr119:
-#line 329 "unicorn_http.rl"
+#line 366 "unicorn_http.rl"
 	{
     VALUE val;
 
@@ -2473,7 +2510,7 @@ tr119:
     if (!STR_CSTR_EQ(val, "*"))
       rb_hash_aset(hp->env, g_path_info, val);
   }
-#line 304 "unicorn_http.rl"
+#line 341 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -2491,14 +2528,14 @@ tr119:
   }
 	goto st73;
 tr126:
-#line 323 "unicorn_http.rl"
+#line 360 "unicorn_http.rl"
 	{MARK(start.query, p); }
-#line 324 "unicorn_http.rl"
+#line 361 "unicorn_http.rl"
 	{
     VALIDATE_MAX_URI_LENGTH(LEN(start.query, p), QUERY_STRING);
     rb_hash_aset(hp->env, g_query_string, STR_NEW(start.query, p));
   }
-#line 304 "unicorn_http.rl"
+#line 341 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -2516,12 +2553,12 @@ tr126:
   }
 	goto st73;
 tr131:
-#line 324 "unicorn_http.rl"
+#line 361 "unicorn_http.rl"
 	{
     VALIDATE_MAX_URI_LENGTH(LEN(start.query, p), QUERY_STRING);
     rb_hash_aset(hp->env, g_query_string, STR_NEW(start.query, p));
   }
-#line 304 "unicorn_http.rl"
+#line 341 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -2542,7 +2579,7 @@ st73:
 	if ( ++p == pe )
 		goto _test_eof73;
 case 73:
-#line 2546 "unicorn_http.c"
+#line 2583 "unicorn_http.c"
 	switch( (*p) ) {
 		case 10: goto tr108;
 		case 13: goto tr109;
@@ -2555,14 +2592,14 @@ case 73:
 		goto st0;
 	goto tr107;
 tr107:
-#line 290 "unicorn_http.rl"
+#line 327 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st74;
 st74:
 	if ( ++p == pe )
 		goto _test_eof74;
 case 74:
-#line 2566 "unicorn_http.c"
+#line 2603 "unicorn_http.c"
 	switch( (*p) ) {
 		case 10: goto tr112;
 		case 13: goto tr113;
@@ -2575,14 +2612,14 @@ case 74:
 		goto st0;
 	goto st74;
 tr110:
-#line 290 "unicorn_http.rl"
+#line 327 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st75;
 st75:
 	if ( ++p == pe )
 		goto _test_eof75;
 case 75:
-#line 2586 "unicorn_http.c"
+#line 2623 "unicorn_http.c"
 	if ( (*p) < 65 ) {
 		if ( 48 <= (*p) && (*p) <= 57 )
 			goto st76;
@@ -2606,20 +2643,20 @@ case 76:
 		goto st74;
 	goto st0;
 tr102:
-#line 290 "unicorn_http.rl"
+#line 327 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st77;
 tr147:
-#line 303 "unicorn_http.rl"
+#line 340 "unicorn_http.rl"
 	{ rb_hash_aset(hp->env, g_http_host, STR_NEW(mark, p)); }
-#line 290 "unicorn_http.rl"
+#line 327 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st77;
 st77:
 	if ( ++p == pe )
 		goto _test_eof77;
 case 77:
-#line 2623 "unicorn_http.c"
+#line 2660 "unicorn_http.c"
 	switch( (*p) ) {
 		case 10: goto tr117;
 		case 13: goto tr118;
@@ -2659,7 +2696,7 @@ case 79:
 		goto st77;
 	goto st0;
 tr121:
-#line 329 "unicorn_http.rl"
+#line 366 "unicorn_http.rl"
 	{
     VALUE val;
 
@@ -2675,7 +2712,7 @@ st80:
 	if ( ++p == pe )
 		goto _test_eof80;
 case 80:
-#line 2679 "unicorn_http.c"
+#line 2716 "unicorn_http.c"
 	switch( (*p) ) {
 		case 10: goto tr124;
 		case 13: goto tr125;
@@ -2688,14 +2725,14 @@ case 80:
 		goto st0;
 	goto tr123;
 tr123:
-#line 323 "unicorn_http.rl"
+#line 360 "unicorn_http.rl"
 	{MARK(start.query, p); }
 	goto st81;
 st81:
 	if ( ++p == pe )
 		goto _test_eof81;
 case 81:
-#line 2699 "unicorn_http.c"
+#line 2736 "unicorn_http.c"
 	switch( (*p) ) {
 		case 10: goto tr129;
 		case 13: goto tr130;
@@ -2708,14 +2745,14 @@ case 81:
 		goto st0;
 	goto st81;
 tr127:
-#line 323 "unicorn_http.rl"
+#line 360 "unicorn_http.rl"
 	{MARK(start.query, p); }
 	goto st82;
 st82:
 	if ( ++p == pe )
 		goto _test_eof82;
 case 82:
-#line 2719 "unicorn_http.c"
+#line 2756 "unicorn_http.c"
 	if ( (*p) < 65 ) {
 		if ( 48 <= (*p) && (*p) <= 57 )
 			goto st83;
@@ -2739,58 +2776,58 @@ case 83:
 		goto st81;
 	goto st0;
 tr103:
-#line 290 "unicorn_http.rl"
+#line 327 "unicorn_http.rl"
 	{MARK(mark, p); }
-#line 294 "unicorn_http.rl"
+#line 331 "unicorn_http.rl"
 	{ downcase_char(deconst(p)); }
 	goto st84;
 st84:
 	if ( ++p == pe )
 		goto _test_eof84;
 case 84:
-#line 2752 "unicorn_http.c"
+#line 2789 "unicorn_http.c"
 	switch( (*p) ) {
 		case 84: goto tr134;
 		case 116: goto tr134;
 	}
 	goto st0;
 tr134:
-#line 294 "unicorn_http.rl"
+#line 331 "unicorn_http.rl"
 	{ downcase_char(deconst(p)); }
 	goto st85;
 st85:
 	if ( ++p == pe )
 		goto _test_eof85;
 case 85:
-#line 2766 "unicorn_http.c"
+#line 2803 "unicorn_http.c"
 	switch( (*p) ) {
 		case 84: goto tr135;
 		case 116: goto tr135;
 	}
 	goto st0;
 tr135:
-#line 294 "unicorn_http.rl"
+#line 331 "unicorn_http.rl"
 	{ downcase_char(deconst(p)); }
 	goto st86;
 st86:
 	if ( ++p == pe )
 		goto _test_eof86;
 case 86:
-#line 2780 "unicorn_http.c"
+#line 2817 "unicorn_http.c"
 	switch( (*p) ) {
 		case 80: goto tr136;
 		case 112: goto tr136;
 	}
 	goto st0;
 tr136:
-#line 294 "unicorn_http.rl"
+#line 331 "unicorn_http.rl"
 	{ downcase_char(deconst(p)); }
 	goto st87;
 st87:
 	if ( ++p == pe )
 		goto _test_eof87;
 case 87:
-#line 2794 "unicorn_http.c"
+#line 2831 "unicorn_http.c"
 	switch( (*p) ) {
 		case 58: goto tr137;
 		case 83: goto tr138;
@@ -2798,7 +2835,7 @@ case 87:
 	}
 	goto st0;
 tr137:
-#line 300 "unicorn_http.rl"
+#line 337 "unicorn_http.rl"
 	{
     rb_hash_aset(hp->env, g_rack_url_scheme, STR_NEW(mark, p));
   }
@@ -2807,7 +2844,7 @@ st88:
 	if ( ++p == pe )
 		goto _test_eof88;
 case 88:
-#line 2811 "unicorn_http.c"
+#line 2848 "unicorn_http.c"
 	if ( (*p) == 47 )
 		goto st89;
 	goto st0;
@@ -2895,14 +2932,14 @@ case 93:
 		goto st91;
 	goto st0;
 tr143:
-#line 290 "unicorn_http.rl"
+#line 327 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st94;
 st94:
 	if ( ++p == pe )
 		goto _test_eof94;
 case 94:
-#line 2906 "unicorn_http.c"
+#line 2943 "unicorn_http.c"
 	switch( (*p) ) {
 		case 37: goto st92;
 		case 47: goto tr147;
@@ -2954,14 +2991,14 @@ case 95:
 		goto st0;
 	goto st91;
 tr144:
-#line 290 "unicorn_http.rl"
+#line 327 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st96;
 st96:
 	if ( ++p == pe )
 		goto _test_eof96;
 case 96:
-#line 2965 "unicorn_http.c"
+#line 3002 "unicorn_http.c"
 	switch( (*p) ) {
 		case 37: goto st92;
 		case 47: goto st0;
@@ -3039,14 +3076,14 @@ case 98:
 		goto st0;
 	goto st91;
 tr138:
-#line 294 "unicorn_http.rl"
+#line 331 "unicorn_http.rl"
 	{ downcase_char(deconst(p)); }
 	goto st99;
 st99:
 	if ( ++p == pe )
 		goto _test_eof99;
 case 99:
-#line 3050 "unicorn_http.c"
+#line 3087 "unicorn_http.c"
 	if ( (*p) == 58 )
 		goto tr137;
 	goto st0;
@@ -3066,7 +3103,7 @@ case 100:
 		goto tr152;
 	goto st0;
 tr151:
-#line 339 "unicorn_http.rl"
+#line 376 "unicorn_http.rl"
 	{
     hp->len.chunk = step_incr(hp->len.chunk, (*p), 16);
     if (hp->len.chunk < 0)
@@ -3077,7 +3114,7 @@ st101:
 	if ( ++p == pe )
 		goto _test_eof101;
 case 101:
-#line 3081 "unicorn_http.c"
+#line 3118 "unicorn_http.c"
 	switch( (*p) ) {
 		case 10: goto tr153;
 		case 13: goto st102;
@@ -3094,7 +3131,7 @@ case 101:
 		goto tr152;
 	goto st0;
 tr153:
-#line 368 "unicorn_http.rl"
+#line 405 "unicorn_http.rl"
 	{
     HP_FL_SET(hp, INTRAILER);
     cs = http_parser_en_Trailers;
@@ -3107,7 +3144,7 @@ st123:
 	if ( ++p == pe )
 		goto _test_eof123;
 case 123:
-#line 3111 "unicorn_http.c"
+#line 3148 "unicorn_http.c"
 	goto st0;
 st102:
 	if ( ++p == pe )
@@ -3117,7 +3154,7 @@ case 102:
 		goto tr153;
 	goto st0;
 tr152:
-#line 339 "unicorn_http.rl"
+#line 376 "unicorn_http.rl"
 	{
     hp->len.chunk = step_incr(hp->len.chunk, (*p), 16);
     if (hp->len.chunk < 0)
@@ -3128,7 +3165,7 @@ st103:
 	if ( ++p == pe )
 		goto _test_eof103;
 case 103:
-#line 3132 "unicorn_http.c"
+#line 3169 "unicorn_http.c"
 	switch( (*p) ) {
 		case 10: goto st104;
 		case 13: goto st107;
@@ -3149,7 +3186,7 @@ st104:
 case 104:
 	goto tr159;
 tr159:
-#line 376 "unicorn_http.rl"
+#line 413 "unicorn_http.rl"
 	{
   skip_chunk_data_hack: {
     size_t nr = MIN((size_t)hp->len.chunk, REMAINING);
@@ -3171,7 +3208,7 @@ st105:
 	if ( ++p == pe )
 		goto _test_eof105;
 case 105:
-#line 3175 "unicorn_http.c"
+#line 3212 "unicorn_http.c"
 	switch( (*p) ) {
 		case 10: goto st100;
 		case 13: goto st106;
@@ -3378,30 +3415,30 @@ case 113:
 		goto st113;
 	goto st0;
 tr172:
-#line 296 "unicorn_http.rl"
+#line 333 "unicorn_http.rl"
 	{ MARK(mark, p); }
-#line 298 "unicorn_http.rl"
+#line 335 "unicorn_http.rl"
 	{ write_cont_value(hp, buffer, p); }
 	goto st114;
 tr175:
-#line 298 "unicorn_http.rl"
+#line 335 "unicorn_http.rl"
 	{ write_cont_value(hp, buffer, p); }
 	goto st114;
 tr182:
-#line 296 "unicorn_http.rl"
+#line 333 "unicorn_http.rl"
 	{ MARK(mark, p); }
-#line 297 "unicorn_http.rl"
+#line 334 "unicorn_http.rl"
 	{ write_value(hp, buffer, p); }
 	goto st114;
 tr185:
-#line 297 "unicorn_http.rl"
+#line 334 "unicorn_http.rl"
 	{ write_value(hp, buffer, p); }
 	goto st114;
 st114:
 	if ( ++p == pe )
 		goto _test_eof114;
 case 114:
-#line 3405 "unicorn_http.c"
+#line 3442 "unicorn_http.c"
 	switch( (*p) ) {
 		case 9: goto st115;
 		case 10: goto tr167;
@@ -3430,14 +3467,14 @@ case 114:
 		goto tr169;
 	goto st0;
 tr171:
-#line 296 "unicorn_http.rl"
+#line 333 "unicorn_http.rl"
 	{ MARK(mark, p); }
 	goto st115;
 st115:
 	if ( ++p == pe )
 		goto _test_eof115;
 case 115:
-#line 3441 "unicorn_http.c"
+#line 3478 "unicorn_http.c"
 	switch( (*p) ) {
 		case 9: goto tr171;
 		case 10: goto tr172;
@@ -3449,14 +3486,14 @@ case 115:
 		goto st0;
 	goto tr170;
 tr170:
-#line 296 "unicorn_http.rl"
+#line 333 "unicorn_http.rl"
 	{ MARK(mark, p); }
 	goto st116;
 st116:
 	if ( ++p == pe )
 		goto _test_eof116;
 case 116:
-#line 3460 "unicorn_http.c"
+#line 3497 "unicorn_http.c"
 	switch( (*p) ) {
 		case 10: goto tr175;
 		case 13: goto tr176;
@@ -3469,35 +3506,35 @@ case 116:
 		goto st0;
 	goto st116;
 tr173:
-#line 296 "unicorn_http.rl"
+#line 333 "unicorn_http.rl"
 	{ MARK(mark, p); }
-#line 298 "unicorn_http.rl"
+#line 335 "unicorn_http.rl"
 	{ write_cont_value(hp, buffer, p); }
 	goto st117;
 tr176:
-#line 298 "unicorn_http.rl"
+#line 335 "unicorn_http.rl"
 	{ write_cont_value(hp, buffer, p); }
 	goto st117;
 tr183:
-#line 296 "unicorn_http.rl"
+#line 333 "unicorn_http.rl"
 	{ MARK(mark, p); }
-#line 297 "unicorn_http.rl"
+#line 334 "unicorn_http.rl"
 	{ write_value(hp, buffer, p); }
 	goto st117;
 tr186:
-#line 297 "unicorn_http.rl"
+#line 334 "unicorn_http.rl"
 	{ write_value(hp, buffer, p); }
 	goto st117;
 st117:
 	if ( ++p == pe )
 		goto _test_eof117;
 case 117:
-#line 3496 "unicorn_http.c"
+#line 3533 "unicorn_http.c"
 	if ( (*p) == 10 )
 		goto st114;
 	goto st0;
 tr167:
-#line 363 "unicorn_http.rl"
+#line 400 "unicorn_http.rl"
 	{
     cs = http_parser_first_final;
     goto post_exec;
@@ -3507,7 +3544,7 @@ st124:
 	if ( ++p == pe )
 		goto _test_eof124;
 case 124:
-#line 3511 "unicorn_http.c"
+#line 3548 "unicorn_http.c"
 	goto st0;
 st118:
 	if ( ++p == pe )
@@ -3517,20 +3554,20 @@ case 118:
 		goto tr167;
 	goto st0;
 tr169:
-#line 292 "unicorn_http.rl"
+#line 329 "unicorn_http.rl"
 	{ MARK(start.field, p); }
-#line 293 "unicorn_http.rl"
+#line 330 "unicorn_http.rl"
 	{ snake_upcase_char(deconst(p)); }
 	goto st119;
 tr178:
-#line 293 "unicorn_http.rl"
+#line 330 "unicorn_http.rl"
 	{ snake_upcase_char(deconst(p)); }
 	goto st119;
 st119:
 	if ( ++p == pe )
 		goto _test_eof119;
 case 119:
-#line 3534 "unicorn_http.c"
+#line 3571 "unicorn_http.c"
 	switch( (*p) ) {
 		case 33: goto tr178;
 		case 58: goto tr179;
@@ -3556,18 +3593,18 @@ case 119:
 		goto tr178;
 	goto st0;
 tr181:
-#line 296 "unicorn_http.rl"
+#line 333 "unicorn_http.rl"
 	{ MARK(mark, p); }
 	goto st120;
 tr179:
-#line 295 "unicorn_http.rl"
+#line 332 "unicorn_http.rl"
 	{ hp->s.field_len = LEN(start.field, p); }
 	goto st120;
 st120:
 	if ( ++p == pe )
 		goto _test_eof120;
 case 120:
-#line 3571 "unicorn_http.c"
+#line 3608 "unicorn_http.c"
 	switch( (*p) ) {
 		case 9: goto tr181;
 		case 10: goto tr182;
@@ -3579,14 +3616,14 @@ case 120:
 		goto st0;
 	goto tr180;
 tr180:
-#line 296 "unicorn_http.rl"
+#line 333 "unicorn_http.rl"
 	{ MARK(mark, p); }
 	goto st121;
 st121:
 	if ( ++p == pe )
 		goto _test_eof121;
 case 121:
-#line 3590 "unicorn_http.c"
+#line 3627 "unicorn_http.c"
 	switch( (*p) ) {
 		case 10: goto tr185;
 		case 13: goto tr186;
@@ -3727,7 +3764,7 @@ case 121:
 	_out: {}
 	}
 
-#line 437 "unicorn_http.rl"
+#line 474 "unicorn_http.rl"
 post_exec: /* "_out:" also goes here */
   if (hp->cs != http_parser_error)
     hp->cs = cs;
@@ -4225,7 +4262,7 @@ static VALUE HttpParser_rssget(VALUE self)
 
 void Init_unicorn_http(void)
 {
-  VALUE mUnicorn, cHttpParser;
+  VALUE mUnicorn;
 
   mUnicorn = rb_define_module("Unicorn");
   cHttpParser = rb_define_class_under(mUnicorn, "HttpParser", rb_cObject);
@@ -4285,5 +4322,6 @@ void Init_unicorn_http(void)
 #ifndef HAVE_RB_HASH_CLEAR
   id_clear = rb_intern("clear");
 #endif
+  id_is_chunked_p = rb_intern("is_chunked?");
 }
 #undef SET_GLOBAL
